/**
 * @file
 * VuoList_VuoKey C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoKey_H
#define VuoList_VuoKey_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoKey VuoList_VuoKey
 * A list of @ref VuoKey elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoKey_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoKey elements.
 */
typedef const struct VuoList_VuoKey_struct { void *l; } * VuoList_VuoKey;
/// @}
#endif

/**
 * Creates a new list of @ref VuoKey elements.
 */
VuoList_VuoKey VuoListCreate_VuoKey(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoKey to quickly initialize a large list.
 */
VuoList_VuoKey VuoListCreateWithCount_VuoKey(const unsigned long count, const VuoKey value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoKey VuoListCopy_VuoKey(const VuoList_VuoKey list);

/**
 * Returns the @ref VuoKey at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoKey VuoListGetValue_VuoKey(const VuoList_VuoKey list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoKey *VuoListGetData_VuoKey(const VuoList_VuoKey list);

/**
 * Changes the @ref VuoKey at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoKey(const VuoList_VuoKey list, const VuoKey value, const unsigned long index);

/**
 * Inserts the `VuoKey` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoKey(const VuoList_VuoKey list, const VuoKey value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoKey(VuoList_VuoKey list, const VuoKey value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoKey(VuoList_VuoKey list, const VuoKey value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoKey(VuoList_VuoKey list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoKey(VuoList_VuoKey list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoKey(VuoList_VuoKey list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoKey(VuoList_VuoKey list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoKey(VuoList_VuoKey list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoKey(VuoList_VuoKey list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoKey(VuoList_VuoKey list);

/**
 * Removes the `VuoKey` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoKey(VuoList_VuoKey list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoKey(const VuoList_VuoKey list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoKey VuoList_VuoKey_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoKey_getJson(const VuoList_VuoKey value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoKey_getSummary(const VuoList_VuoKey value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoKey VuoList_VuoKey_makeFromString(const char *str);
char * VuoList_VuoKey_getString(const VuoList_VuoKey value);
/// @}

/**
 * @}
 */

#endif
