/**
 * @file
 * Logging functions.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOLOG_H
#define VUOLOG_H

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @ingroup DevelopingNodeClasses DevelopingTypes DevelopingLibraryModules
 * @defgroup VuoModuleDebug Module Debugging
 * Macros to help with debugging.
 *
 * @{
 */

double VuoLogGetTime(void);
double VuoLogGetElapsedTime(void);
bool VuoIsDebugEnabled(void);
char *VuoLog_copyCFDescription(const void *variable);

/**
 * Outputs a message to the system log and to `stderr`.
 *
 * Also stores the most recent 2 messages in the OS X CrashReporter data structure, to be included with crash reports.
 */
#ifdef DOXYGEN
void VuoLog(const char *file, const unsigned int line, const char *function, const char *format, ...);
#else
void VuoLog(const char *file, const unsigned int line, const char *function, const char *format, ...) __attribute__((format(printf, 4, 5)));
#endif

/**
 * Prints the name of the file and function to the system log and to `stderr` (and implicitly flushes the output buffer).
 *
 * \eg{
 * void nodeEvent()
 * {
 *     VL();
 * }
 * }
 *
 * @hideinitializer
 */
#define VL() VuoLog(__FILE__, __LINE__, __func__, "")

/**
 * Prints the name of the file and function, and `printf()`-style format/arguments, to the system log and to `stderr` (and implicitly flushes the output buffer).
 *
 * Use this for debugging only; don't commit files with `VLog` calls.  To log messages intended for end users, see @ref VUserLog.
 *
 * \eg{
 * void nodeEvent(VuoInputData(VuoInteger, "42") number)
 * {
 *     VLog("%d", number);
 * }
 * }
 *
 * @hideinitializer
 */
#define VLog(format, ...) VuoLog(__FILE__, __LINE__, __func__, format, ##__VA_ARGS__)

/**
 * Same as @ref VLog, but for messages intended for end users.
 *
 * @hideinitializer
 */
#define VUserLog(format, ...) VuoLog(__FILE__, __LINE__, __func__, format, ##__VA_ARGS__)

/**
 * Prints the name of the current file and function, and the address and description of the specified `heapPointer`, to the system log and to `stderr` (and implicitly flushes the output buffer).
 *
 * \eg{
 * void nodeEvent(VuoInputData(VuoShader) shader)
 * {
 *     VLogHeap(shader);
 * }
 * }
 *
 * @hideinitializer
 */
#define VLogHeap(heapPointer) do { \
	char *description = VuoHeap_getDescription(heapPointer); \
	VLog("%s = %p (registered at %s)", #heapPointer, heapPointer, description); \
	free(description); \
	} while(0)

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif // VUOLOG_H
